@@ -23,7 +23,7 @@
 #include "kconfigini_p.h"
 
 #include "kconfig.h"
-#include "kconfigbackend.h"
+#include "kconfigbackend_p.h"
 #include "bufferfragment_p.h"
 #include "kconfigdata.h"
 
@@ -46,21 +46,22 @@ KCONFIGCORE_EXPORT bool kde_kiosk_exception = false; // flag to disable kiosk re
 
 static QByteArray lookup(const KConfigIniBackend::BufferFragment &fragment, QHash<KConfigIniBackend::BufferFragment, QByteArray> *cache)
 {
-    QHash<KConfigIniBackend::BufferFragment, QByteArray>::iterator it = cache->find(fragment);
-    if (it == cache->end()) {
-        it = cache->insert(fragment, fragment.toByteArray());
+    auto it = cache->constFind(fragment);
+    if (it != cache->constEnd()) {
+        return it.value();
     }
-    return it.value();
+
+    return cache->insert(fragment, fragment.toByteArray()).value();
 }
 
 QString KConfigIniBackend::warningProlog(const QFile &file, int line)
 {
-    return QString::fromLatin1("KConfigIni: In file %2, line %1: ")
+    return QStringLiteral("KConfigIni: In file %2, line %1: ")
            .arg(line).arg(file.fileName());
 }
 
 KConfigIniBackend::KConfigIniBackend()
-    : KConfigBackend(), lockFile(Q_NULLPTR)
+    : KConfigBackend(), lockFile(nullptr)
 {
 }
 
@@ -165,7 +166,7 @@ KConfigIniBackend::parseConfig(const QByteArray &currentLocale, KEntryMap &entry
             } while ((start = end + 2) <= line.length() && line.at(end + 1) == '[');
             currentGroup = newGroup;
 
-            groupSkip = entryMap.getEntryOption(currentGroup, 0, 0, KEntryMap::EntryImmutable);
+            groupSkip = entryMap.getEntryOption(currentGroup, nullptr, nullptr, KEntryMap::EntryImmutable);
 
             if (groupSkip && !bDefault) {
                 continue;
@@ -192,13 +193,14 @@ KConfigIniBackend::parseConfig(const QByteArray &currentLocale, KEntryMap &entry
                 temp.trim();
                 aKey = temp;
                 line.truncateLeft(eqpos + 1);
+                line.trim();
             }
             if (aKey.isEmpty()) {
                 qWarning() << warningProlog(file, lineNo) << "Invalid entry (empty key)";
                 continue;
             }
 
-            KEntryMap::EntryOptions entryOptions = 0;
+            KEntryMap::EntryOptions entryOptions = nullptr;
             if (groupOptionImmutable) {
                 entryOptions |= KEntryMap::EntryImmutable;
             }
@@ -599,14 +601,8 @@ void KConfigIniBackend::setFilePath(const QString &file)
     const QFileInfo info(file);
     if (info.exists()) {
         setLocalFilePath(info.canonicalFilePath());
-        setLastModified(info.lastModified());
-        setSize(info.size());
     } else {
         setLocalFilePath(file);
-        setSize(0);
-        QDateTime dummy;
-        dummy.setTime_t(0);
-        setLastModified(dummy);
     }
 }
 
@@ -639,7 +635,7 @@ void KConfigIniBackend::unlock()
 {
     lockFile->unlock();
     delete lockFile;
-    lockFile = Q_NULLPTR;
+    lockFile = nullptr;
 }
 
 bool KConfigIniBackend::isLocked() const
@@ -680,6 +676,11 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray &aString, Strin
             if (((unsigned char)s[i]) < 32) {
                 goto doEscape;
             }
+            // GroupString and KeyString should be valid UTF-8, but ValueString
+            // can be a bytearray with non-UTF-8 bytes that should be escaped.
+            if (type == ValueString && ((unsigned char)s[i]) >= 127) {
+                goto doEscape;
+            }
             *data++ = s[i];
             break;
         case '\n':
@@ -726,7 +727,6 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray &aString, Strin
     if (result.endsWith(' ') && type != GroupString) {
         result.replace(result.length() - 1, 1, "\\s");
     }
-    result.squeeze();
 
     return result;
 }
@@ -798,6 +798,12 @@ void KConfigIniBackend::printableToString(BufferFragment *aString, const QFile &
                 r++;
                 *r = ';';
                 break;
+            case ',':
+                // not really an escape sequence, but allowed in .desktop files, don't strip '\,' from the string
+                *r = '\\';
+                r++;
+                *r = ',';
+                break;
             case 'x':
                 if (i + 2 < l) {
                     *r = charFromHex(str + i + 1, file, line);
@@ -810,7 +816,7 @@ void KConfigIniBackend::printableToString(BufferFragment *aString, const QFile &
             default:
                 *r = '\\';
                 qWarning() << warningProlog(file, line)
-                           << QString::fromLatin1("Invalid escape sequence \"\\%1\".").arg(str[i]);
+                           << QStringLiteral("Invalid escape sequence \"\\%1\".").arg(str[i]);
             }
         }
     }
